pro mk_ssc,  week,  outdir=outdir, loud=loud, refresh=refresh, $
  noscatter=noscatter, scatter=scatter, nodeka=nodeka, nokleen=nokleen, $
  vignette=vignette, novignette=novignette, $
  nodestreak=nodestreak, destreak=destreak, $
  noclean=noclean, noleak=noleak, no_second_order_leak=no_second_order_leak, $
  prefix=prefix,  ss=ss, triplet=triplet, nonormalize=nonormalize, $
  saa_state=saa_state, ifiles=ifiles, outfile=outfile, $
  subdir=subdir, yyyy=yyyy, filename_only=filename_only, _extra=_extra, $
  uncert=uncert

;+
;   Name: mk_ssc
;
;   Purpose: make SXT "Final" science composite movie - de-spagetti mk_sfd
;
;   Input Parameters:
;      week - the week to process 
; 
;   Output Parameters:
;     NONE
;
;   Keyword Parameters:
;      nodeka     - dont deka normalize
;      novignette - dont vignette correct
;      noclean    - dont call sxt_clean (despike and optional)
;      noscatter  - dont call sxt_scatter
;
;      prefix - output file prefix (default='ssc' (pair), 'sst' implies triple 
;      ifiles - explicit filelist (in lieu of WEEK string)
;      subdir - optional subdirectory 
;      filename_only - if set, just derive planned filename and return in OUTFILE
;  
;   Side Effects:
;      generates an 'ssc' file - floating point, normalized to counts/sec/pix
;   
;   History:
;      14-Sep-1998 - S.L.Freeland  - Written (YPOP/MSU)
;      16-Sep-1998 - Various mods - include proc flags in file name.
;      17-Sep-1998 - Add scattered light correction and /NOSCATTER
;      18-Dec-1998 - Change normalization-> from deka-norm->counts/sec/pixel
;                    Change default prefix to SSC (Sxt Science Composite)
;                    Triplet->                SST 
;      21-Mar-1999 - S.L.Freeland - change /sxt_kleen->sxt_clean
;                    Add /destreak (sxt_xsaa)hook/logic
;
;      21-apr-1999 - S.L.Freeland - allow filelist in lieu of week string
;      12-aug-1999 - S.L.Freeland - add /YYYY,  /YYYYMM, and /YYYYMMDD keywords
;                    (subdirectory formats)
;      29-Sep-1999 - S.L.Freeland - add /FILENAME_ONLY keyword and function
;      26-Oct-1999 - S.L.Freeland - add /NOLEAK ( -> sxt_prep )
;      18-Jan-2001 - S.L.Freeland - add /UNCERT
;  
;-
common mk_ssc_blk1, lweek, files, roadmap

mk_ssc_version='.1'  
noleak=keyword_set(noleak)

; Processing Flags (boolean in file name of form xxxx (0=>off,1=>on)
; ORDER>>       SCATTER | VIGNETTE | CLEAN | DESTREAK | NORMALIZE | 2ndLEAK

scatter=keyword_set(scatter)               ; Default OFF
vignette=keyword_set(vignette)             ; Default OFF        
destreak=keyword_set(destreak)             ; Default OFF
soleak=1-keyword_set(no_second_order_leak) ; Default ON
clean=(1-keyword_set(noclean)) or destreak ; Default ON

deka=0                                     ; S.L.F 18-dec-1998 Default Changed
normalize=1-keyword_set(nonormalize)       ; Default ON

;------- make the processing flags part of the file name ------------
flagstring=strcompress(string(scatter)+ $
  string(vignette)+string(clean)+string(destreak)+string(normalize) + $
  string(soleak),/remove)

; ------------ define output directory ----------------
defdir=get_logenv('DIR_SXT_SSC')
case 1 of
  data_chk(outdir,/string,/scalar):                ; user supplied
  file_exist(defdir):              outdir =defdir  ; online
  else: outdir=curdir()                            ; current directory
endcase

if not keyword_set(outdir) then outdir=curdir()
case 1 of
   data_chk(prefix,/string,/scalar):                ; user supplied
   keyword_set(triplet):             prefix='sst'   ; triplet movie
   else:                             prefix='ssc'   ; default (pair) movie
endcase   

case 1 of
   keyword_set(saa_state): prefix=prefix + 'saa' ; saa only
   n_elements(saa_state) eq 1: prefix=prefix + 'nosaa'
   else:
endcase   

loud=keyword_set(loud)

if n_elements(lweek) eq 0 then lweek=''

; -------- read roadmaps ------------

case 1 of
   data_chk(week,/string): begin
     box_message,'Checking CDROM for associated week files...'
     files=cdrom_files(week=week,/sfr) 
     froot=week
   endcase
   data_chk(ifiles,/string): begin
      box_message,'Using user supplied filelist...'     
      files=ifiles
      froot=time2file(file2time(ifiles(0))) 
      case 1 of
         keyword_set(yyyy): subdir=strmid(froot,0,4)     ; prepend ../YYYY/...
	 data_chk(subdir,/string): sub=subdir
         else: subdir=''
      endcase
      outdir=concat_dir(outdir,subdir)
      box_message,'Using parent directory: ' + outdir
      week=''
   endcase
   else: begin
      box_message,'Need to supply either week (YYYY_WW) or file list (IFILES=filelist)'
      return
   endcase
endcase   

outfile=concat_dir(outdir,prefix+froot+'_'+flagstring+ mk_ssc_version)

if keyword_set(filename_only) then begin
   box_message,'Just returing filename in OUTFILE...'
   return
endif   


states=(['DISABLED','ENABLED'])
status=['Output File Name: ' + outfile,      $
  'Processing Options:' ,                    $
  '   Scattered Light:     ' + states(scatter), $
  '   Vignette Correction :' + states(vignette),$
  '   SXT_Clean (despike) :' + states(clean),   $
  '   SXT_XSAA  (destreak):' + states(destreak),   $
  '   Normal->Cts/Sec/Pix: ' +       states(normalize), $
  '   2nd Order Leak Corr: ' +       states(soleak) ]

if noleak then begin
   box_message,'LEAK SUBTRACTION INHIBITED!',btext=txt,/quiet
   status=[txt,status]
endif 

box_message,status

rd_roadmap,files,roadmap
lweek=week

; -------- make LONG/SHORT pariings ----------
if loud then box_message,'Calling image-paring routine...'
case 1 of 
   keyword_set(triplet): begin
      mk_sff_pair, roadmap, ssl, ssm, sss, /triplet, delta=delta, saa_state=saa_state ,count=count,_extra=_extra  ; triplets
      readstr='rd_sda,files,[ssl(i),ssm(i),sss(i)],index,data'    ; for later
   endcase
   else: begin
      mk_sff_pair, roadmap, ssl, sss, delta=delta,saa_state=saa_state,count=count,_extra=_extra    ; doublets
      readstr='rd_sda,files,[ssl(i),sss(i)],index,data'           ; for later
   endcase
endcase
;

nlong=n_elements(ssl)
if n_elements(ss) ne 0 then nlong=ss
if count eq 0 then nlong=0

if loud then box_message,'Number of pairs found: ' + strtrim(nlong,2)

for i=0,nlong(0) -1 do begin
  delvarx, data, fillmap
  exstat=execute(readstr)                       ; read index,data via execute

; ---------- dont composite if not required ---------
  satlong=sxt_satscreen(index(0),data(*,*,0))   
  
  if not satlong then begin

     if loud then box_message,['Compositing Image Pair:', $
	 get_info(roadmap(ssl(i)),/non),get_info(roadmap(sss(i)),/non)]

;    ---- Call compositing routine (calls sxt_prep internally) ---
     sxt_sff_composite, index, data, cindex, cdata, $
        vignette=vignette, sxt_cleanx=clean, destreak=destreak, noleak=noleak, $
        second_order_leak=soleak, fillmap=fillmap

   endif else begin
     if loud then box_message,['No compositing required for: ', $
	 get_info(roadmap(ssl(i)),/non)]

;      ----- No compositing needed - just call sxt_prep ---------
       sxt_prep, index(0),data(*,*,0), cindex, cdata, $
          /float,/dc_orbit_correct,vignette=vignette, $
          sxt_cleanx=clean, destreak=destreak, noleak=noleak, $
          second_order_leak=soleak
       fillmap=bytarr(data_chk(data,/nx),data_chk(data,/ny))
   endelse

;  --------------- scatter correction ------------------   
   if scatter then begin 
      box_message,'Removing scatter'
      sxt_scatter2, cindex, cdata, scatindex, scatdata
      cindex=scatindex
      cdata=float(double(cdata)-double(scatdata))
   endif

;  ----------- normalization ---------------
   case 1 of
      normalize: begin                               ; -> counts/sec/pixel
         box_message,'Normalizing to CTS/Sec/Pixl'
	 for j=0,data_chk(cdata,/nimage)-1 do $
	      cdata(0,0,j) = 1.e3*cdata(*,*,j)/gt_expdur(cindex(j))
	 his_index, cindex, -1, 'expdur', 1000.                     ; history
      endcase

      deka: begin
         box_message,'DEKA-Normalizing'       
         cdata=deka_norm(cindex, cdata, deka=cindex)       
      endcase
      else: box_messge,'Not Normalizing data'
   endcase

   if keyword_set(uncert) then begin     ; encode uncertainty info
      ss1=where(fillmap eq 1,ss1cnt)
      ss2=where(fillmap eq 2,ss2cnt)
      cindex.his.spare(0)=gt_dpe(index(0))
      if ss1cnt gt 0 then begin 
         cdata(ss1)=cdata(ss1)+2.^30.
         cindex(ss1).his.spare(1)=gt_dpe(index(1))
      endif
      if ss2cnt gt 0 then begin 
         cdata(ss2)=cdata(ss2)+2.^31.
         cindex(ss2).his.spare(2)=gt_dpe(index(2))
      endif
   endif
   sav_sda,outfile,cindex,cdata,append=(i ne 0)
endfor  

return
end

